#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

typedef struct
{
  char *type;
  char *diskpath;
  char *resopath;
  size_t offset;
  size_t size;
} refi_t;

refi_t refi[2000];
int n_refi;

// restool input file format:
// # comment
// type file-path-on-disk file-path-in-resources

static FILE *open_read(const char *fn)
{
  FILE *f= fopen(fn,"rb");
  if (!f) exit(fprintf(stderr,"can't open %s: %s\n", fn, strerror(errno)));
  return f;
}

static FILE *open_write(const char *fn)
{
  FILE *f= fopen(fn,"wb");
  if (!f) exit(fprintf(stderr,"can't open %s: %s\n", fn, strerror(errno)));
  return f;
}

static void parse_input(char*fn)
{
  int i;
  FILE *f;
  char g[2000];
  f= open_read(fn);
  while(fgets(g,sizeof(g),f))
  {
    g[sizeof(g)-1]= 0;
    for(i=0;g[i];i++) if (g[i]=='\r' || g[i]=='\n') { g[i]= 0; break; }
    for(i=0;isspace(g[i]);i++) ;
    if (!g[i]) continue;
    if (g[i]=='#') continue;
    memmove(g, g+i, strlen(g+i)+1);
    char typ[2000],dpa[2000],rpa[2000];
    sscanf(g,"%s%s%s",typ,dpa,rpa);
    printf("type=[%s] dpath=[%s] rpath=[%s]\n",typ,dpa,rpa);
    refi[n_refi].type= strdup(typ);
    refi[n_refi].diskpath= strdup(dpa);
    refi[n_refi].resopath= strdup(rpa);
    n_refi++;
  }
  fclose(f);
}

void prt_str(size_t offset, FILE *s, unsigned char* g, int R)
{
  int j;
      for(j=0;j<R;j++)
      {
        if (offset%12==0)
          fprintf(s,"\n   ");
        fprintf(s,"0x%02x,", g[j]);
        offset++;
      }
}

int main(int argc,char **argv)
{
  int i;
  FILE *h,*s;
  parse_input(argv[1]);

  h= open_write("../lib/bltfile.hh");
  fprintf(h,"const char *get_builtin_file(const char *name,size_t *Rsize,const char **Rtype);\n");
  fprintf(h,"const char *list_builtin_file(int *Rstart,const char *type);\n");

  fclose(h);

  s= open_write("../lib/bltfile.cc");

  fprintf(s,"#include <stdlib.h>\n");
  fprintf(s,"#include <string.h>\n");
  fprintf(s,"#include \"bltfile.hh\"\n");
  fprintf(s,"typedef struct { const char *type,*data,*path; size_t size; } blt_t;\n");

  fprintf(s,"static const char data[]={");
  size_t offset= 0;
  for(i=0;i<n_refi;i++)
  {
    unsigned char g[4000];
    int R;
    FILE *f;
    refi[i].offset= offset;
    refi[i].size= 0;
    f= open_read(refi[i].diskpath);
    while(1)
    {
      R= fread(g,1,sizeof(g), f);
      if (R<=0) break;
      prt_str(offset,s,g, R);
      offset+= R;
      refi[i].size+= R;
    }
    prt_str(offset, s, (unsigned char*) "", 1);
    offset++;
    fclose(f);
  }

  fprintf(s,"\n};\n");

  fprintf(s,"static const blt_t blt[]= {\n");
  for(i=0;i<n_refi;i++)
    fprintf(s,"  { \"%s\", data+%d, \"%s\", %d },\n",
      refi[i].type, refi[i].offset, refi[i].resopath, refi[i].size);
  fprintf(s,"  { NULL, NULL, NULL, 0 }\n");
  fprintf(s,"};\n");

  fprintf(s,"const char *get_builtin_file(const char *name,size_t *Rsize,const char **Rtype){\n");
  fprintf(s,"  int i;\n");
  fprintf(s,"  for(i=0;blt[i].path;i++)\n");
  fprintf(s,"     if (!strcmp(name,blt[i].path))\n");
  fprintf(s,"     { if (Rsize) *Rsize= blt[i].size; \n");
  fprintf(s,"       if (Rtype) *Rtype= blt[i].type; \n");
  fprintf(s,"       return blt[i].data; } \n");
  fprintf(s,"  if (Rtype) *Rtype= NULL;\n");
  fprintf(s,"  if (Rsize) *Rsize= 0;\n");
  fprintf(s,"  return NULL;\n");
  fprintf(s,"}\n");
  fprintf(s,"const char *list_builtin_file(int *Rstart,const char *type){\n");
  fprintf(s,"  int i;\n");
  fprintf(s,"  if (*Rstart<=0) *Rstart= 0;\n");
  fprintf(s,"  if (*Rstart>=%d) return NULL;\n",n_refi);
  fprintf(s,"  for(i=*Rstart;blt[i].path;i++)\n");
  fprintf(s,"   if ((type && !strcmp(blt[i].type,type)) || !type)\n");
  fprintf(s,"    { *Rstart= i+1; return blt[i].path; }\n");
  fprintf(s,"  *Rstart= %d;\n",n_refi);
  fprintf(s,"  return NULL;\n");
  fprintf(s,"}\n");

  fclose(s);
  return 0;
}
